describe McDump::Memcached::Server do

  let(:connection_args) { { host: "some.host", port: 12345 } }

  let(:server) { described_class.new(connection_args) }

  describe "#items" do

    let(:stats_dump)     { "some stats dump" }
    let(:items)          { (1..3).map { (1..3).map { instance_double(McDump::Memcached::Item) } } }
    let(:stats)          { (1..3).map { instance_double(McDump::Memcached::Stat) } }
    let(:telnet_session) { instance_double(McDump::Telnet::Session, stats: stats_dump) }

    subject { server.items }

    before(:example) do
      allow(McDump::Telnet::Session).to receive(:open).and_yield(telnet_session)
      allow(McDump::Memcached::Stat).to receive(:parse).and_return(stats)
      stats.zip(items) { |stat, items| allow(stat).to receive(:items).and_return(items) }
    end

    it "opens a telnet session using the provided connection arguments" do
      expect(McDump::Telnet::Session).to receive(:open).with(connection_args)

      subject
    end

    it "retrieves the stats via the Memcached telnet session" do
      expect(telnet_session).to receive(:stats)

      subject
    end

    it "parses the stats" do
      expect(McDump::Memcached::Stat).to receive(:parse).with(stats_dump)

      subject
    end

    it "retrieves the items for each stat" do
      stats.each { |stat| expect(stat).to receive(:items).with(telnet_session) }

      subject
    end

    it "returns all the items" do
      expect(subject).to eql(items.flatten)
    end

  end

end
