describe McDump::Memcached::Stat do

  let(:items_id) { 8 }
  let(:number)   { 21 }

  let(:stat) { described_class.new(items_id: items_id, number: number) }

  describe "::parse" do

    subject { described_class.parse(dump) }

    context "when the dump contains many items" do

      let(:dump) do
        <<-DUMP
STAT items:9:number 19
STAT items:9:age 876543
STAT items:9:evicted 0
STAT items:9:evicted_nonzero 0
STAT items:9:evicted_time 0
STAT items:9:outofmemory 0
STAT items:9:tailrepairs 0
STAT items:9:reclaimed 3
STAT items:9:expired_unfetched 2
STAT items:9:evicted_unfetched 0
STAT items:9:crawler_reclaimed 0
STAT items:9:crawler_items_checked 0
STAT items:9:lrutail_reflocked 0
STAT items:10:number 20
STAT items:10:age 123456
STAT items:10:evicted 0
STAT items:10:evicted_nonzero 0
STAT items:10:evicted_time 0
STAT items:10:outofmemory 0
STAT items:10:tailrepairs 0
STAT items:10:reclaimed 6
STAT items:10:expired_unfetched 8
STAT items:10:evicted_unfetched 0
STAT items:10:crawler_reclaimed 0
STAT items:10:crawler_items_checked 0
STAT items:10:lrutail_reflocked 0
STAT items:11:number 21
STAT items:11:age 975319
STAT items:11:evicted 0
STAT items:11:evicted_nonzero 0
STAT items:11:evicted_time 0
STAT items:11:outofmemory 0
STAT items:11:tailrepairs 0
STAT items:11:reclaimed 1
STAT items:11:expired_unfetched 10
STAT items:11:evicted_unfetched 0
STAT items:11:crawler_reclaimed 0
STAT items:11:crawler_items_checked 0
STAT items:11:lrutail_reflocked 0
        DUMP
      end

      it "creates a stat for each item entry containing the number of entries in the cache" do
        stats_arg_expectations = [
          { items_id: "9", number: "19" }, { items_id: "10", number: "20" }, { items_id: "11", number: "21" }
        ]
        stats_arg_expectations.each { |expectation| expect(described_class).to receive(:new).with(expectation) }

        subject
      end

      it "returns the stats" do
        stats = (1..3).map { instance_double(described_class) }
        allow(described_class).to receive(:new).and_return(*stats)

        expect(subject).to eql(stats)
      end

    end

    context "when the dump is empty" do

      let(:dump) { "" }

      it "returns an empty array" do
        expect(subject).to eql([])
      end

    end

  end

  describe "#items" do

    let(:items_dump)      { "some items data" }
    let(:telnet_session) { instance_double(McDump::Telnet::Session, items: items_dump) }

    subject { stat.items(telnet_session) }

    before(:example) { allow(telnet_session).to receive(:items).and_return(items_dump) }

    it "determines the items in the memcached instance via the telnet session" do
      expect(telnet_session).to receive(:items).with(items_id, number)

      subject
    end

    it "parses the items returned via telnet" do
      expect(McDump::Memcached::Item).to receive(:parse).with(items_dump, items_id)

      subject
    end

    it "returns the parsed items" do
      items = (1..3).map { instance_double(McDump::Memcached::Item) }
      allow(McDump::Memcached::Item).to receive(:parse).and_return(items)

      expect(subject).to eql(items)
    end

  end

end
