describe McDump::Rake::Task do

  describe "constructor" do

    let(:name)               { :some_dump_task }
    let(:host)               { "some.host" }
    let(:port)               { "12345" }
    let(:timeout_in_seconds) { "10" }
    let(:connection_args)    { {} }

    subject { described_class.new(name, connection_args) }

    after(:example) { ::Rake::Task[name].clear }

    it "creates a rake task with the provided name" do
      subject

      expect(::Rake::Task.task_defined?(name)).to be(true)
    end

    describe "the created rake task" do

      subject do
        described_class.new(name, connection_args)

        ::Rake::Task[name].execute
      end

      context "when connection arguments are provided" do

        let(:connection_args) { { host: host, port: port, timeout_in_seconds: timeout_in_seconds } }

        it "dumps a server identified by the provided connection arguments" do
          expect(McDump).to receive(:server).with(connection_args)

          subject
        end

      end

      context "when no connection arguments are provided" do

        let(:connection_args) { {} }

        before(:example) do
          @original_env_host               = ENV["host"]
          @original_env_port               = ENV["port"]
          @original_env_timeout_in_seconds = ENV["timeout_in_seconds"]

          ENV["host"]               = host
          ENV["port"]               = port
          ENV["timeout_in_seconds"] = timeout_in_seconds
        end

        after(:example) do
          ENV["host"]               = @original_env_host
          ENV["port"]               = @original_env_port
          ENV["timeout_in_seconds"] = @original_env_timeout_in_seconds
        end

        it "dumps a server identified by environment variables host, port and timeout_in_seconds" do
          expect(McDump).to receive(:server).with(host: host, port: port, timeout_in_seconds: timeout_in_seconds)

          subject
        end

      end

    end

  end

end
