describe McDump::Report do

  let(:report) { described_class.new(items) }

  describe "#dump" do

    let(:io)     { StringIO.new }
    let(:output) { io.string }

    subject { report.dump(io) }

    context "when multiple items are provided" do

      let(:base_time) { Time.new(2015, 11, 17, 12, 0, 0, "+10:00") }

      let(:items) do
        (1..3).map do |i|
          McDump::Memcached::Item.new(
            items_id: "#{i}",
            cache_key: "some long cache key #{i}",
            expiration_time: base_time + i,
            size_in_bytes: i * 1024)
        end
      end

      it "outputs a header row containing each column header spaced to display values" do
        expected_header_row = "| items_id |             cache_key |           expiration_time | size_in_bytes |"

        subject

        expect(output).to start_with(expected_header_row)
      end

      it "outputs a data row for each item" do
        expected_data_rows =
          "|        1 | some long cache key 1 | 2015-11-17 12:00:01 +1000 |          1024 |\n" +
          "|        2 | some long cache key 2 | 2015-11-17 12:00:02 +1000 |          2048 |\n" +
          "|        3 | some long cache key 3 | 2015-11-17 12:00:03 +1000 |          3072 |"

        subject

        expect(output).to include(expected_data_rows)
      end

    end

    context "when no items are provided" do

      let(:items) { [] }

      it "outputs an empty header row" do
        subject

        expect(output).to eql("||\n")
      end

    end

  end

end