describe McDump::Telnet::Connection do

  let(:host)            { "some.host" }
  let(:port)            { 12345 }
  let(:additional_args) { {} }
  let(:args)            { { host: host, port: port }.merge(additional_args) }
  let(:telnet)          { instance_double(Net::Telnet) }

  let(:connection) { described_class.new(args) }

  before(:example) { allow(Net::Telnet).to receive(:new).and_return(telnet) }

  describe "constructor" do

    subject { connection }

    it "creates a telnet instance with the provided host and port" do
      expect(Net::Telnet).to receive(:new).with(hash_including("Host" => host, "Port" => port))

      subject
    end

    context "when a connection timeout is provided" do

      let(:timeout_in_seconds) { 8 }
      let(:additional_args)    { { timeout_in_seconds: timeout_in_seconds } }

      it "creates a telnet instance with the provided timeout in seconds" do
        expect(Net::Telnet).to receive(:new).with(hash_including("Timeout" => timeout_in_seconds))

        subject
      end

    end

    context "when a connection timeout is not provided" do

      let(:additional_args) { {} }

      it "defaults the timeout to 5 seconds" do
        expect(Net::Telnet).to receive(:new).with(hash_including("Timeout" => 5))

        subject
      end

    end

  end

  describe "#execute" do

    let(:command) { "some command" }

    subject { connection.execute(command) }

    it "sends the provided command string to the underlying telnet instance" do
      expect(telnet).to receive(:cmd).with(hash_including("String" => command))

      subject
    end

    it "concludes awaiting output when END is observed in the stream" do
      expect(telnet).to receive(:cmd).with(hash_including("Match" => /^END/))

      subject
    end

    it "returns the result of the command" do
      allow(telnet).to receive(:cmd).and_return("some command result")

      expect(subject).to eql("some command result")
    end

  end

  describe "#close" do

    subject { connection.close }

    it "closes the underlying telnet instance" do
      expect(telnet).to receive(:close)

      subject
    end

  end

end
